'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var BetterBatch = function () {
    function BetterBatch(docClient) {
        var maxRetries = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 10;
        var backoffFunction = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : _helpers.backoff;

        _classCallCheck(this, BetterBatch);

        this.docClient = docClient;
        this.maxRetries = maxRetries;
        this.backoffFunction = backoffFunction;
    }

    _createClass(BetterBatch, [{
        key: 'processChunk',
        value: function processChunk(chunk, operation, unprocessedCheck, callback) {
            var _this = this;

            var retryCount = 0;
            var shouldRetry = true;

            var allItems = new Map();

            _async2.default.whilst(function () {
                return shouldRetry;
            }, function (callback) {
                _this.docClient[operation](chunk, function (err, data) {
                    if (err) return callback(err);

                    // Only used if this is a batchGet
                    if (data.Responses) {
                        var tables = Object.keys(data.Responses);

                        tables.forEach(function (table) {
                            if (!allItems.has(table)) allItems.set(table, []);

                            var itemsOnTable = allItems.get(table);

                            itemsOnTable = itemsOnTable.concat(data.Responses[table]);

                            allItems.set(table, itemsOnTable);
                        });
                    }

                    var unprocessed = unprocessedCheck(data);

                    if (unprocessed) {
                        chunk.RequestItems = unprocessed;
                        if (retryCount < _this.maxRetries) {
                            shouldRetry = true;
                            retryCount += 1;
                        } else {
                            shouldRetry = false;
                            return callback();
                        }

                        return setTimeout(function () {
                            return callback();
                        }, _this.backoffFunction(retryCount));
                    }

                    shouldRetry = false;
                    callback();
                });
            }, function (err) {
                if (err) return callback(err);

                // Note allItems is just an empty map if this is a write op, or the get returned no items
                callback(null, allItems);
            });
        }
    }, {
        key: 'batchOperation',
        value: function batchOperation(operation, chunks, unprocessedCheck, callback) {
            var _this2 = this;

            var allItems = new Map();

            _async2.default.eachSeries(chunks, function (chunk, callback) {
                _this2.processChunk(chunk, operation, unprocessedCheck, function (err, itemMap) {
                    if (err) return callback(err);

                    // Item map empty if it's a write or the get returned nothing
                    itemMap.forEach(function (items, table) {
                        if (!allItems.has(table)) allItems.set(table, []);

                        var currentItems = allItems.get(table);

                        currentItems = currentItems.concat(items);

                        allItems.set(table, currentItems);
                    });

                    callback();
                });
            }, function (err) {
                if (err) return callback(err);

                var mapAsLiteral = {};

                allItems.forEach(function (items, table) {
                    mapAsLiteral[table] = items;
                });

                callback(null, mapAsLiteral);
            });
        }
    }, {
        key: 'batchGet',
        value: function batchGet(params, callback) {
            var chunks = (0, _helpers.chunkItems)(params, 100, true);

            this.batchOperation('batchGet', chunks, function (data) {
                if (Object.keys(data.UnprocessedKeys).length) {
                    return data.UnprocessedKeys;
                }
            }, callback);
        }
    }, {
        key: 'batchWrite',
        value: function batchWrite(params, callback) {
            var chunks = (0, _helpers.chunkItems)(params, 25);

            this.batchOperation('batchWrite', chunks, function (data) {
                if (Object.keys(data.UnprocessedItems).length) {
                    return data.UnprocessedItems;
                }
            }, function (err) {
                // We don't just pass callback as a function argument because batchOperation
                // calls back with an error and an empty map if we're doing a write
                callback(err);
            });
        }
    }, {
        key: 'query',
        value: function query(params, callback) {
            var _this3 = this;

            var queryAgain = true;
            var items = [];

            _async2.default.whilst(function () {
                return queryAgain;
            }, function (callback) {
                _this3.docClient.query(params, function (err, data) {
                    if (err) return callback(err);

                    items = items.concat(data.Items);

                    queryAgain = !!data.LastEvaluatedKey;
                    params.ExclusiveStartKey = data.LastEvaluatedKey;

                    callback();
                });
            }, function (err) {
                if (err) return callback(err);

                callback(null, items);
            });
        }
    }]);

    return BetterBatch;
}();

module.exports = BetterBatch;